@@ -30,6 +30,20 @@ typedef struct {
     SV *(*compile_to_end)(char *, char *, U32 *pos);
 } perl6_callbacks;
 
+#ifndef wrap_keyword_plugin
+void wrap_keyword_plugin(pTHX_ Perl_keyword_plugin_t new_plugin, Perl_keyword_plugin_t *old_plugin_p)
+{
+    dVAR;
+
+    PERL_UNUSED_CONTEXT;
+    if (*old_plugin_p) return;
+    if (!*old_plugin_p) {
+        *old_plugin_p = PL_keyword_plugin;
+        PL_keyword_plugin = new_plugin;
+    }
+}
+#endif
+
 XS(p5_call_p6_method);
 XS(p5_call_p6_extension_method);
 XS(p5_destroy_p5_object);
@@ -230,7 +244,7 @@ U32 p5_sv_utf8(PerlInterpreter *my_perl, SV* sv) {
         char * const pv  = SvPV(sv, len);
         STRLEN i;
         for (i = 0; i < len; i++)
-            if (pv[i] < 0) // signed char!
+            if ((signed char) pv[i] < 0) // signed char!
                 return 0;
         return 1;
     }
@@ -789,7 +803,7 @@ SV *p5_call_inherited_package_method(PerlInterpreter *my_perl, char *package, ch
         push_arguments(sp, len, args);
 
         GV * const gv = p5_look_up_package_method(my_perl, base_package, name, 0);
-        SV * const rv = sv_2mortal(newRV_inc((SV*)GvCV(gv)));
+        SV * const rv = GvCV(gv) ? sv_2mortal(newRV_inc((SV*)GvCV(gv))) : (SV*)gv;
         *count = call_sv(rv, flags);
 
         SPAGAIN;
@@ -835,7 +849,7 @@ SV *p5_call_gv(PerlInterpreter *my_perl, GV *gv, int len, SV *args[], I32 *count
 
         PUTBACK;
 
-        SV * const rv = sv_2mortal(newRV((SV*)GvCV(gv))); /* FIXME: can be done once */
+        SV * const rv = GvCV(gv) ? sv_2mortal(newRV_inc((SV*)GvCV(gv))) : (SV*)gv;
 
         *count = call_sv(rv, G_ARRAY | G_EVAL);
         SPAGAIN;
@@ -939,7 +953,7 @@ SV *p5_scalar_call_gv(PerlInterpreter *my_perl, GV *gv, int len, SV *args[], I32
 
         PUTBACK;
 
-        SV * const rv = sv_2mortal(newRV((SV*)GvCV(gv))); /* FIXME: can be done once */
+        SV * const rv = GvCV(gv) ? sv_2mortal(newRV_inc((SV*)GvCV(gv))) : (SV*)gv;
 
         *count = call_sv(rv, G_SCALAR | G_EVAL);
         SPAGAIN;
@@ -981,7 +995,7 @@ SV *p5_scalar_call_parent_gv(PerlInterpreter *my_perl, GV *gv, int len, SV *args
 
         PUTBACK;
 
-        SV * const rv = sv_2mortal(newRV((SV*)GvCV(gv))); /* FIXME: can be done once */
+        SV * const rv = GvCV(gv) ? sv_2mortal(newRV_inc((SV*)GvCV(gv))) : (SV*)gv;
 
         *count = call_sv(rv, G_SCALAR | G_EVAL);
         SPAGAIN;
@@ -1015,7 +1029,7 @@ SV *p5_call_gv_two_args(PerlInterpreter *my_perl, GV *gv, SV *arg, SV *arg2, I32
 
         PUTBACK;
 
-        SV * const rv = sv_2mortal(newRV((SV*)GvCV(gv)));
+        SV * const rv = GvCV(gv) ? sv_2mortal(newRV_inc((SV*)GvCV(gv))) : (SV*)gv;
 
         *count = call_sv(rv, G_ARRAY | G_EVAL);
         SPAGAIN;
@@ -1053,7 +1067,7 @@ SV *p5_scalar_call_gv_two_args(PerlInterpreter *my_perl, GV *gv, SV *arg, SV *ar
 
         PUTBACK;
 
-        SV * const rv = sv_2mortal(newRV((SV*)GvCV(gv)));
+        SV * const rv = GvCV(gv) ? sv_2mortal(newRV_inc((SV*)GvCV(gv))) : (SV*)gv;
 
         *count = call_sv(rv, G_SCALAR | G_EVAL);
         SPAGAIN;
@@ -1084,7 +1098,7 @@ SV *p5_call_method(PerlInterpreter *my_perl, SV *obj, I32 context, char *name, i
         SAVETMPS;
 
         HV * const pkg = SvSTASH((SV*)SvRV(obj));
-        GV * const gv = Perl_gv_fetchmethod_autoload(aTHX_ pkg, name, TRUE);
+        GV * const gv = gv_fetchmethod_autoload(pkg, name, TRUE);
         if (gv && isGV(gv)) {
             PUSHMARK(SP);
 
@@ -1101,7 +1115,7 @@ SV *p5_call_method(PerlInterpreter *my_perl, SV *obj, I32 context, char *name, i
 
             PUTBACK;
 
-            SV * const rv = sv_2mortal(newRV((SV*)GvCV(gv)));
+            SV * const rv = GvCV(gv) ? sv_2mortal(newRV_inc((SV*)GvCV(gv))) : (SV*)gv;
 
             *count = call_sv(rv, flags);
             SPAGAIN;
@@ -1134,7 +1148,7 @@ SV *p5_call_parent_method(PerlInterpreter *my_perl, char *package, SV *parent_ob
         SAVETMPS;
 
         HV * const pkg = package != NULL ? gv_stashpv(package, 0) : SvSTASH((SV*)SvRV(parent_obj));
-        GV * const gv = Perl_gv_fetchmethod_autoload(aTHX_ pkg, name, TRUE);
+        GV * const gv = gv_fetchmethod_autoload(pkg, name, TRUE);
         if (gv && isGV(gv)) {
             SV * obj;
 
@@ -1155,7 +1169,7 @@ SV *p5_call_parent_method(PerlInterpreter *my_perl, char *package, SV *parent_ob
 
             PUTBACK;
 
-            SV * const rv = sv_2mortal(newRV((SV*)GvCV(gv)));
+            SV * const rv = GvCV(gv) ? sv_2mortal(newRV_inc((SV*)GvCV(gv))) : (SV*)gv;
 
             *count = call_sv(rv, flags);
             SPAGAIN;
